#!/usr/bin/env python
# coding: utf-8

# In[1]:


import msprime,pyslim
import tskit
import numpy as np
import subprocess
import sys


# In[ ]:


### Overview ###

# this code combines SLiM3 with MSPrime to simulate adaptive introgression sweeps.
# it generates the necessary input files and performs a VolcanoFinder scan of the data

# the simForward.eidos script must be present in the working directory

# the parameters for this simulation run are defined internally, below


# In[130]:


### arguments ###

#itt = sys.argv[1] # unique identifier for this simulation run
itt = 1 # placeholder for script to run without passing simulation id


### Simulation Parameters ###


alpha = 1000 
T_donor = 2 #units of 4N generations
T_outgroup = 10 #units of 4N generations


L = 1e6
Ne = 1e4 #population size
r = 5e-7 #recombination rate per site
u = 1.25e-7 #mutation rate per site
m = 1/Ne #migration rate 



### Parameters for runing VolcanoFinder ###


n = 40 #sample size
gridSpace = 1e5 #didstance between test sites
D = -1 #i.e. computes grid of D values 1 theta, 2 theta... k theta
polarized = 1 # i.e. fixed diffs are polarized
model = 1 # i.e. use model 1 of volcanofinder

fileTempTree = f"/tmp/tmpTree_{itt}.trees" #temporary slim tree-sequence generated for conditioning on fixation
fileSlimTree = f"slimTree_{itt}.trees" # slim tree-sequence output
fileFreqFile = f"freqFile_{itt}.txt" #  frequency file for VolcanoFinder
fileSpectFile = f"spectFile_{itt}.txt" #  spectrum file for VolcanoFinder
fileScanFile = f"vfScan_{itt}.txt" #  scan reults file from VolcanoFinder

vf = "/home/derek/Desktop/vf_longChr/volcanofinder_v1.0/VolcanoFinder" #path to VolcanoFinder
slim = 'slim' #path to slim




### convert parameters for use in simulators ###
sB = alpha/(2*Ne)
Tdonor = 4 * Ne * T_donor
Toutgroup = 4 * Ne *T_outgroup
L = int(L)
Ne = int(Ne)
offset=int(offset)

#### extra parameters #####
offset = 0 
# ben allele is positioned at position L/2 by default. 
# This adjusts the base pair position left or right if desired 





# In[121]:


#### runs forward simulations in slim ####

# user must provide the path to run slim

# simForward.eidos file must be in same directory
# this contains the code to run forward simulations

    slim,
    '-d', f'simID={itt}',
    '-d', f's_b={sB}',
    '-d', f'Ne={Ne}',
    '-d', f'r={r}',
    '-d', f'm={m}',
    '-d', f'L={L}',
    '-d', f'offset={offset}',
    '-d', f'tempFile="{fileTempTree}"',
    '-d', f'outFile="{fileSlimTree}"',
    'simForward.eidos' # must be in the same directory
]

proc=subprocess.check_output(slimProc)


# In[122]:


### defines the model for backward simulations in msprime ###

ts = pyslim.load(fileSlimTree)
population_configurations = [
    msprime.PopulationConfiguration(initial_size = 1), # empty population (required for slim tree-sequences)
    msprime.PopulationConfiguration(initial_size = Ne), #recipient population size
    msprime.PopulationConfiguration(initial_size = 1), #donor population size
    msprime.PopulationConfiguration(initial_size = 1) #ougroup population size
migration_matrix = [
    [0,0,0,0],
    [0,0,0,0],
    [0,0,0,0],
    [0,0,0,0]
]
demographic_events = [
    msprime.MassMigration(time=Tdonor,source = 2, destination = 1, proportion = 1.0),
    msprime.MassMigration(time=Toutgroup, source = 3, destination = 1, proportion=1.0)
]

### backward simulations using msprime ###

recap = ts.recapitate(
    recombination_rate=r,
    population_configurations=population_configurations,
    migration_matrix = migration_matrix,
    demographic_events=demographic_events
)

## this permits saving the complete tree-sequence generated from the simulations ##
# recap.dump(out_file_name) #must provide out file name


# In[124]:


### adds mutations to the tree and samples n individual chromosomes ####

mut_ts = msprime.mutate(tree_sequence=recap,rate=u)

testSample = list(ts.samples(population = 1)[0:n]) ##sample


# In[125]:


### generates SFS file for VF ###

spectPlus = mut_ts.allele_frequency_spectrum(
    sample_sets = [testSample],
    mode='site',
    polarised=True,
    span_normalise=False)

spectrum = [x /mut_ts.sequence_length for x in spectPlus[1:]]

with open(fileSpectFile,"w") as outfile:
    for i,x in enumerate(spectrum):
        outfile.write(f"{i+1}\t{x}\n")


# In[126]:


### generates input data file for VF ###

with open(fileFreqFile,"w") as outfile:
    outfile.write("position\tx\tn\tfolded\n")
    folded = 0
    for variant in mut_ts.variants(samples=testSample):
        count = sum(variant.genotypes)
        pos = variant.site.position
        if count == 0: continue;
        outfile.write(f"{pos}\t{count}\t{n}\t{folded}\n")


# In[131]:


### runs volcanofinder scan  ###

# must provide path to run VolcanoFinder

vfProc = [
    vf,
    '-ig', f'{gridSpace}',
    fileFreqFile,
    fileSpectFile,
    f'{D}',
    f'{polarized}',
    f'{model}',
    f'{fileScanFile}'
]
proc = subprocess.check_output(vfProc)


# In[ ]:




